import('model.Cursor')
import('model.Screen')
import('model.events.CursorStackChanged')
import('model.events.InventoryChanged')
import('model.Inventory')
import('model.toolbar.Toolbar')

---@class Interface : Object
---@field private _player LuaPlayer
---@field private _screen LuaPlayer
Interface = Object:extendAs("application.Interface")
Interface.__instances = {}

---@public
---@param player_index number
---@return Interface
function Interface:getInstanceFor(player_index)
    if Interface.__instances[player_index] == nil then
        Interface.__instances[player_index] = Interface:new(player_index)
    end
    return Interface.__instances[player_index]
end

---@public
---@return Interface
function Interface:new(player_index)
    local this = Interface:super(Object:new())
    this._player = game.players[player_index]
    this._screen = Screen:new({}, this._player.gui.screen)
    return this
end

---@private
---@return Screen
function Interface:screen()
    return self._screen
end

---@public
function Interface:createToolbar()
    self:screen():createToolbar():centerOnScreen()
end

---@public
function Interface:deleteAll()
    for _, toolbar in ipairs(self:screen():children()) do
        toolbar:delete()
    end
end

---@public
---@param guiClickEvent GuiClickEvent
function Interface:handleGuiClick(guiClickEvent)
    if guiClickEvent:modName() == "interface-mod" then
        self:screen():handleGuiClick(guiClickEvent)
    end
end

---@public
---@param guiChangedEvent GuiEvent
function Interface:handleGuiChange(guiChangedEvent)
    if guiChangedEvent:modName() == "interface-mod" then
        self:screen():handleGuiChange(guiChangedEvent)
    end
end

---@public
---@param event Event
function Interface:handle(event)
    if event:isInstanceOf(InventoryChanged) then
        Inventory:invalidateFor(self._player.index)
    end
    self:screen():handle(event)
end
