import('model.Component')
import('model.toolbar.content.sections.section.content.table.Row')

---@class Table : Component
Table = Component:extendAs("model.toolbar.content.sections.section.content.table.Table")

function Table:create(parent)
    return Component.create(self, parent, {
        type = "frame",
        direction = "vertical",
        style = "toolbar_content_sections_section_content_table" })
end

function Table:new(parent, root)
    local instance = Table:super(Component:new(parent, root))
    instance:loadChildren()
    return instance
end

function Table:childrenClasses()
    return { Row }
end

---@public
---@param width number
---@param minimumHeight number
---@param heightPadding number
function Table:resize(width, minimumHeight, heightPadding)
    self:setHeight(minimumHeight, heightPadding)
    self:setWidth(width)
end

---@private
---@param minimumHeight number
---@param heightPadding number
function Table:setHeight(minimumHeight, heightPadding)
    self:trimHeight(minimumHeight)
    self:pad(minimumHeight, heightPadding)
end

---@private
function Table:trimHeight()
    local lastOccupiedRowIndex = self:lastOccupiedRowIndex()
    for _, row in ipairs(self:rows()) do
        if row:index() > lastOccupiedRowIndex then
            row:delete()
        end
    end
end

---@private
---@return number
function Table:lastOccupiedRowIndex()
    local lastOccupiedRowIndex = 0
    for _, row in ipairs(self:rows()) do
        if row:isOccupied() then
            lastOccupiedRowIndex = math.max(lastOccupiedRowIndex, row:index())
        end
    end
    return lastOccupiedRowIndex
end

---@private
---@param minimumHeight number
---@param padding number
function Table:pad(minimumHeight, padding)
    local missingRows = minimumHeight - self:height()
    local rowsToAdd = missingRows > 0 and missingRows or padding
    local i = 0
    while i < rowsToAdd do
        Row:create(self)
        i = i + 1
    end
end

---@private
---@param width number
function Table:setWidth(width)
    for _, row in ipairs(self:rows()) do
        row:setWidth(width)
    end
end

---@public
---@return number
function Table:lastOccupiedColumnIndex()
    local rows = self:rows()
    local lastOccupiedColumnIndex = 0
    for _, row in ipairs(rows) do
        lastOccupiedColumnIndex = math.max(lastOccupiedColumnIndex, row:lastOccupiedSlotIndex())
    end
    return lastOccupiedColumnIndex
end

---@private
---@return number
function Table:height()
    return #self:root().children
end

---@private
---@return Row[]
function Table:rows()
    return self:children()
end
