import('model.Component')
import('model.toolbar.Toolbar')
import('model.toolbar.content.sections.section.header.MoveDown')
import('model.toolbar.content.sections.section.header.MoveUp')
import('model.toolbar.content.sections.section.header.ExpandSection')
import('model.toolbar.content.sections.section.header.CollapseSection')
import('model.toolbar.content.sections.section.header.DeleteSection')
import('model.toolbar.content.sections.section.header.CancelDeleteSection')
import('model.toolbar.content.sections.section.header.ConfirmDeleteSection')
import('model.toolbar.content.sections.section.header.SectionName')

---@class SectionHeader : Component
SectionHeader = Component:extendAs("model.toolbar.content.sections.section.header.Header")

function SectionHeader:create(parent)
    local instance = Component.create(self, parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_content_sections_section_header" })
    instance:root().style.bottom_margin = 1

    MoveDown:create(instance)
    MoveUp:create(instance)
    SectionName:create(instance):focus()
    CollapseSection:create(instance)
    DeleteSection:create(instance)

    return instance
end

function SectionHeader:new(parent, root)
    local instance = SectionHeader:super(Component:new(parent, root))
    instance:loadChildren()
    return instance
end

function SectionHeader:childrenClasses()
    return {
        MoveDown, MoveUp,
        CollapseSection, ExpandSection,
        SectionName,
        DeleteSection, ConfirmDeleteSection, CancelDeleteSection }
end

---@public
function SectionHeader:askForDeletion()
    self:childOfType(DeleteSection):delete()
    ConfirmDeleteSection:create(self)
    CancelDeleteSection:create(self)
end

---@public
function SectionHeader:cancelDeletion()
    self:childOfType(CancelDeleteSection):delete()
    self:childOfType(ConfirmDeleteSection):delete()
    DeleteSection:create(self)
end

function SectionHeader:lock()
    if self:toolbar():sectionsCount() == 1 then
        self:hide()
    end
    Component.lock(self)
end

function SectionHeader:unlock()
    self:show()
    Component.unlock(self)
end

function SectionHeader:toolbar()
    return self:ancestor(Toolbar)
end

function SectionHeader:name()
    return self:childOfType(SectionName):text()
end
