import('model.Component')
import('model.toolbar.Toolbar')
import('model.toolbar.header.Lock')
import('model.toolbar.header.Unlock')
import('model.toolbar.header.CollapseToolbar')
import('model.toolbar.header.ExpandToolbar')
import('model.toolbar.header.DeleteToolbar')
import('model.toolbar.header.CancelDeleteToolbar')
import('model.toolbar.header.ConfirmDeleteToolbar')

---@class ToolbarHeader : Component
ToolbarHeader = Component:extendAs("model.toolbar.header.Header")

function ToolbarHeader:create(parent)
    local instance = Component.create(self, parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_header" })
    Lock:create(instance)
    local drag = instance:root().add {
        type = "empty-widget",
        style = "toolbar_drag" }
    drag.drag_target = instance:ancestor(Toolbar):root()

    CollapseToolbar:create(instance)
    DeleteToolbar:create(instance)
    return instance
end

function ToolbarHeader:new(parent, root)
    local instance = ToolbarHeader:super(Component:new(parent, root))
    instance:loadChildren()
    return instance
end

function ToolbarHeader:childrenClasses()
    return { Lock, Unlock,
             CollapseToolbar, ExpandToolbar,
             DeleteToolbar, CancelDeleteToolbar, ConfirmDeleteToolbar }
end

function ToolbarHeader:askForDeletion()
    self:childOfType(DeleteToolbar):delete()
    ConfirmDeleteToolbar:create(self)
    CancelDeleteToolbar:create(self)
end

function ToolbarHeader:cancelDeletion()
    self:childOfType(CancelDeleteToolbar):delete()
    self:childOfType(ConfirmDeleteToolbar):delete()
    local deleteToolbar = DeleteToolbar:create(self)
    if self:isLocked() then
        deleteToolbar:lock()
    end
end

function ToolbarHeader:isLocked()
    return #self:childrenOfType(Unlock) > 0
end
